      module hyperbolics_module

      public :: asinh, acosh, atanh

      contains

      FUNCTION asinh (X)
!***BEGIN PROLOGUE  ASINH
!***PURPOSE  Compute the arc hyperbolic sine.
!***LIBRARY   SLATEC (FNLIB)
!***CATEGORY  C4C
!***TYPE      SINGLE PRECISION (ASINH-S, DASINH-D, CASINH-C)
!***KEYWORDS  ARC HYPERBOLIC SINE, ASINH, ELEMENTARY FUNCTIONS, FNLIB,
!             INVERSE HYPERBOLIC SINE
!***AUTHOR  Fullerton, W., (LANL)
!***DESCRIPTION
!
! ASINH(X) computes the arc hyperbolic sine of X.
!
! Series for ASNH       on the interval  0.          to  1.00000D+00
!                                        with weighted error   2.19E-17
!                                         log weighted error  16.66
!                               significant figures required  15.60
!                                    decimal places required  17.31
!
!***REFERENCES  (NONE)
!***ROUTINES CALLED  CSEVL, INITS, R1MACH
!***REVISION HISTORY  (YYMMDD)
!   770401  DATE WRITTEN
!   890531  Changed all specific intrinsics to generic.  (WRB)
!   890531  REVISION DATE from Version 3.2
!   891214  Prologue converted to Version 4.0 format.  (BAB)
!***END PROLOGUE  ASINH
      DIMENSION ASNHCS(20)
      LOGICAL FIRST
      SAVE ALN2, ASNHCS, NTERMS, XMAX, SQEPS, FIRST
      DATA ALN2 /0.69314718055994530942E0/
      DATA ASNHCS( 1) /   -.12820039911738186E0 /
      DATA ASNHCS( 2) /   -.058811761189951768E0 /
      DATA ASNHCS( 3) /    .004727465432212481E0 /
      DATA ASNHCS( 4) /   -.000493836316265361E0 /
      DATA ASNHCS( 5) /    .000058506207058557E0 /
      DATA ASNHCS( 6) /   -.000007466998328931E0 /
      DATA ASNHCS( 7) /    .000001001169358355E0 /
      DATA ASNHCS( 8) /   -.000000139035438587E0 /
      DATA ASNHCS( 9) /    .000000019823169483E0 /
      DATA ASNHCS(10) /   -.000000002884746841E0 /
      DATA ASNHCS(11) /    .000000000426729654E0 /
      DATA ASNHCS(12) /   -.000000000063976084E0 /
      DATA ASNHCS(13) /    .000000000009699168E0 /
      DATA ASNHCS(14) /   -.000000000001484427E0 /
      DATA ASNHCS(15) /    .000000000000229037E0 /
      DATA ASNHCS(16) /   -.000000000000035588E0 /
      DATA ASNHCS(17) /    .000000000000005563E0 /
      DATA ASNHCS(18) /   -.000000000000000874E0 /
      DATA ASNHCS(19) /    .000000000000000138E0 /
      DATA ASNHCS(20) /   -.000000000000000021E0 /
      DATA FIRST /.TRUE./
!***FIRST EXECUTABLE STATEMENT  ASINH
      IF (FIRST) THEN
         NTERMS = INITS (ASNHCS, 20, 0.1*R1MACH(3))
         SQEPS = SQRT (R1MACH(3))
         XMAX = 1.0/SQEPS
      ENDIF
      FIRST = .FALSE.
!
      Y = ABS(X)
      IF (Y.GT.1.0) GO TO 20
!
      ASINH = X
      IF (Y.GT.SQEPS) ASINH = X*(1.0 + CSEVL (2.*X*X-1., ASNHCS,NTERMS))
      RETURN
!
 20   IF (Y.LT.XMAX) ASINH = LOG (Y + SQRT(Y**2+1.))
      IF (Y.GE.XMAX) ASINH = ALN2 + LOG(Y)
      ASINH = SIGN (ASINH, X)
!
      RETURN
      END function asinh

      FUNCTION acosh (X)
!***BEGIN PROLOGUE  ACOSH
!***PURPOSE  Compute the arc hyperbolic cosine.
!***LIBRARY   SLATEC (FNLIB)
!***CATEGORY  C4C
!***TYPE      SINGLE PRECISION (ACOSH-S, DACOSH-D, CACOSH-C)
!***KEYWORDS  ACOSH, ARC HYPERBOLIC COSINE, ELEMENTARY FUNCTIONS, FNLIB,
!             INVERSE HYPERBOLIC COSINE
!***AUTHOR  Fullerton, W., (LANL)
!***DESCRIPTION
!
! ACOSH(X) computes the arc hyperbolic cosine of X.
!
!***REFERENCES  (NONE)
!***ROUTINES CALLED  R1MACH, XERMSG
!***REVISION HISTORY  (YYMMDD)
!   770401  DATE WRITTEN
!   890531  Changed all specific intrinsics to generic.  (WRB)
!   890531  REVISION DATE from Version 3.2
!   891214  Prologue converted to Version 4.0 format.  (BAB)
!   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
!   900326  Removed duplicate information from DESCRIPTION section.
!           (WRB)
!***END PROLOGUE  ACOSH
      SAVE ALN2,XMAX
      DATA ALN2 / 0.69314718055994530942E0/
      DATA XMAX /0./
!***FIRST EXECUTABLE STATEMENT  ACOSH
      IF (XMAX.EQ.0.) XMAX = 1.0/SQRT(R1MACH(3))
!
      IF (X .LT. 1.0) &
         CALL XERMSG ('SLATEC', 'ACOSH', 'X LESS THAN 1', 1, 2)
!
      IF (X.LT.XMAX) ACOSH = LOG (X + SQRT(X*X-1.0))
      IF (X.GE.XMAX) ACOSH = ALN2 + LOG(X)
!
      RETURN
      END function acosh

      FUNCTION atanh (X)
!***BEGIN PROLOGUE  ATANH
!***PURPOSE  Compute the arc hyperbolic tangent.
!***LIBRARY   SLATEC (FNLIB)
!***CATEGORY  C4C
!***TYPE      SINGLE PRECISION (ATANH-S, DATANH-D, CATANH-C)
!***KEYWORDS  ARC HYPERBOLIC TANGENT, ATANH, ELEMENTARY FUNCTIONS,
!             FNLIB, INVERSE HYPERBOLIC TANGENT
!***AUTHOR  Fullerton, W., (LANL)
!***DESCRIPTION
!
! ATANH(X) computes the arc hyperbolic tangent of X.
!
! Series for ATNH       on the interval  0.          to  2.50000D-01
!                                        with weighted error   6.70E-18
!                                         log weighted error  17.17
!                               significant figures required  16.01
!                                    decimal places required  17.76
!
!***REFERENCES  (NONE)
!***ROUTINES CALLED  CSEVL, INITS, R1MACH, XERMSG
!***REVISION HISTORY  (YYMMDD)
!   770401  DATE WRITTEN
!   890531  Changed all specific intrinsics to generic.  (WRB)
!   890531  REVISION DATE from Version 3.2
!   891214  Prologue converted to Version 4.0 format.  (BAB)
!   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)
!   900326  Removed duplicate information from DESCRIPTION section.
!           (WRB)
!***END PROLOGUE  ATANH
      DIMENSION ATNHCS(15)
      LOGICAL FIRST
      SAVE ATNHCS, NTERMS, DXREL, SQEPS, FIRST
      DATA ATNHCS( 1) /    .094395102393195492E0 /
      DATA ATNHCS( 2) /    .049198437055786159E0 /
      DATA ATNHCS( 3) /    .002102593522455432E0 /
      DATA ATNHCS( 4) /    .000107355444977611E0 /
      DATA ATNHCS( 5) /    .000005978267249293E0 /
      DATA ATNHCS( 6) /    .000000350506203088E0 /
      DATA ATNHCS( 7) /    .000000021263743437E0 /
      DATA ATNHCS( 8) /    .000000001321694535E0 /
      DATA ATNHCS( 9) /    .000000000083658755E0 /
      DATA ATNHCS(10) /    .000000000005370503E0 /
      DATA ATNHCS(11) /    .000000000000348665E0 /
      DATA ATNHCS(12) /    .000000000000022845E0 /
      DATA ATNHCS(13) /    .000000000000001508E0 /
      DATA ATNHCS(14) /    .000000000000000100E0 /
      DATA ATNHCS(15) /    .000000000000000006E0 /
      DATA FIRST /.TRUE./
!***FIRST EXECUTABLE STATEMENT  ATANH
      IF (FIRST) THEN
         NTERMS = INITS (ATNHCS, 15, 0.1*R1MACH(3))
         DXREL = SQRT (R1MACH(4))
         SQEPS = SQRT (3.0*R1MACH(3))
      ENDIF
      FIRST = .FALSE.
!
      Y = ABS(X)
      IF (Y .GE. 1.0) &
         CALL XERMSG ('SLATEC', 'ATANH', 'ABS(X) GE 1', 2, 2)
!
      IF (1.0-Y .LT. DXREL) CALL XERMSG ('SLATEC', 'ATANH', &
         'ANSWER LT HALF PRECISION BECAUSE ABS(X) TOO NEAR 1', 1, 1)
!
      ATANH = X
      IF (Y.GT.SQEPS .AND. Y.LE.0.5) &
         ATANH = X*(1.0 + CSEVL (8.*X*X-1., ATNHCS, NTERMS))
      IF (Y.GT.0.5) ATANH = 0.5*LOG((1.0+X)/(1.0-X))
!
      RETURN
      END function atanh

      END module hyperbolics_module
